from typing import List, Dict, Callable, Union

from bluer_options.terminal import show_usage, xtra


def help_(
    tokens: List[str],
    mono: bool,
    plugin_name: str = "bluer_ai",
) -> str:
    options = xtra(
        "what=all|<test-name>,dryrun",
        mono=mono,
    )

    test_options = xtra(
        "dryrun",
        mono=mono,
    )

    callable = f"{plugin_name} test"

    if plugin_name == "bluer_ai":
        options = f"{options},plugin=<plugin-name>"
        callable = "@test"

    return show_usage(
        callable.split(" ")
        + [
            f"[{options}]",
            f"[{test_options}]",
        ],
        f"test {plugin_name}.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
    plugin_name: str = "bluer_ai",
) -> str:
    options = "list"

    callable = f"{plugin_name} test"

    if plugin_name == "bluer_ai":
        options = f"{options},plugin=<plugin-name>"
        callable = "@test"

    return show_usage(
        callable.split(" ")
        + [
            f"{options}",
        ],
        f"list {plugin_name} tests.",
        mono=mono,
    )


def help_functions(
    plugin_name: str = "bluer_ai",
) -> Union[Callable, Dict[str, Union[Callable, Dict]]]:
    return {
        "": lambda tokens, mono: help_(
            tokens,
            mono=mono,
            plugin_name=plugin_name,
        ),
        "list": lambda tokens, mono: help_list(
            tokens,
            mono=mono,
            plugin_name=plugin_name,
        ),
    }



from typing import List

from bluer_options.terminal import show_usage


def help_get_module_name(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@plugins",
            "get_module_name",
            "<repo-name>",
        ],
        "get module name for <repo-name>.",
        mono=mono,
    )


def help_install(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@plugins",
            "install",
            "[all | <plugin-name>]",
        ],
        "install plugin(s).",
        mono=mono,
    )


def help_list_of_external(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@plugins",
            "list_of_external",
        ],
        "show list of external plugins.",
        mono=mono,
    )


def help_list_of_installed(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@plugins",
            "list_of_installed",
        ],
        "show list of installed plugins.",
        mono=mono,
    )


def help_transform(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@plugins",
            "transform",
            "<repo-name>",
        ],
        "transform a blue-plugin git clone to <repo-name>.",
        mono=mono,
    )


help_functions = {
    "get_module_name": help_get_module_name,
    "install": help_install,
    "list_of_external": help_list_of_external,
    "list_of_installed": help_list_of_installed,
    "transform": help_transform,
}



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_download(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "filename=<filename>"

    open_options = "open,QGIS"

    return show_usage(
        [
            "@download",
            f"[{options}]",
            "[.|<object-name>]",
            f"[{open_options}]",
        ],
        "download object.",
        mono=mono,
    )



from typing import Dict
from tqdm import tqdm
import glob
import os

from blueness import module
from bluer_objects import file
from bluer_objects.env import abcli_path_git

from bluer_ai import NAME
from bluer_ai.logger import logger

NAME = module.name(__file__, NAME)


def transform(
    repo_name: str,
    plugin_name: str,
    log: bool = True,
    verbose: bool = False,
) -> bool:
    logger.info(
        "{}.transform: {} @ {}".format(
            NAME,
            repo_name,
            plugin_name,
        )
    )

    repo_path = os.path.join(abcli_path_git, repo_name)
    logger.info(f"repo_path: {repo_path}")

    alias_name: str = ""
    if repo_name.startswith("blue-"):
        alias_name = "@{}".format(repo_name.split("blue-", 1)[1])
        logger.info(f"alias: {alias_name}")

    transformation: Dict[str, str] = {
        "blue_plugin": plugin_name,
        "blue-plugin": repo_name,
        "BLUE_PLUGIN": plugin_name.upper(),
    }
    if alias_name:
        transformation["@plugin"] = alias_name
    logger.info(f"{len(transformation)} transformation(s)")
    for index, (this, that) in enumerate(transformation.items()):
        logger.info(f"#{index: 2d} - {this} -> {that}")

    list_of_extensions = "env,py,sh,yml".split(",")

    for root, _, list_of_filenames in tqdm(os.walk(repo_path)):
        for filename in list_of_filenames:
            if any(
                filename.endswith(f".{extension}") for extension in list_of_extensions
            ):
                full_filename = os.path.join(root, filename)

                success, content = file.load_text(
                    full_filename,
                    log=verbose,
                )
                if not success:
                    return False

                for this, that in transformation.items():
                    content = [line.replace(this, that) for line in content]

                success = file.save_text(
                    full_filename,
                    content,
                    log=log,
                )
                if not success:
                    return False

    return True



from typing import List

from bluer_options.terminal import show_usage

example_list = "<item-1>,<item-2>,..."


def help_list_filter(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--contains <this>]",
        "[--doesnt_contain <that>]",
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "filter",
            f"{example_list}",
        ]
        + args,
        "filter list.",
        mono=mono,
    )


def help_list_in(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "in",
            "<item>",
            f"{example_list}",
        ]
        + args,
        "True | False.",
        mono=mono,
    )


def help_list_intersect(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "intersect",
            f"{example_list}",
            f"{example_list}",
        ]
        + args,
        "intersect the two lists.",
        mono=mono,
    )


def help_list_item(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "item",
            f"{example_list}",
            "<index>",
        ]
        + args,
        "list[<index>].",
        mono=mono,
    )


def help_list_len(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "len",
            f"{example_list}",
        ]
        + args,
        "len(list).",
        mono=mono,
    )


def help_list_log(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--before <loading>]",
        "[--after <thing(s)>]",
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "log",
            f"{example_list}",
        ]
        + args,
        "log list.",
        mono=mono,
    )


def help_list_next(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "next",
            "<item>",
            f"{example_list}",
        ]
        + args,
        "item after <item> in list.",
        mono=mono,
    )


def help_list_nonempty(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "nonempty",
            f"{example_list}",
        ]
        + args,
        "non-empty items in list.",
        mono=mono,
    )


def help_list_prev(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "prev",
            "<item>",
            f"{example_list}",
        ]
        + args,
        "item before <item> in list.",
        mono=mono,
    )


def help_list_resize(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@list",
            "resize",
            f"{example_list}",
            "-1 | <length>",
        ]
        + args,
        "resize list.",
        mono=mono,
    )


def help_list_sort(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space | <delim>]",
        "[--unique 0|1]",
    ]

    return show_usage(
        [
            "@list",
            "sort",
            f"{example_list}",
        ]
        + args,
        "sortlist.",
        mono=mono,
    )


help_functions = {
    "filter": help_list_filter,
    "in": help_list_in,
    "intersect": help_list_intersect,
    "item": help_list_item,
    "len": help_list_len,
    "log": help_list_log,
    "next": help_list_next,
    "nonempty": help_list_nonempty,
    "prev": help_list_prev,
    "resize": help_list_resize,
    "sort": help_list_sort,
}



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_cat(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@cat",
            "<filename>",
        ],
        "cat <filename>.",
        mono=mono,
    )


def help_log(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@log",
            "<message>",
        ],
        "log message.",
        mono=mono,
    )


def help_log_list(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        '[--before "list of"]',
        '[--after "items(s)"]',
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "abcli_log_list",
            "<this,that>",
        ]
        + args,
        "log list.",
        mono=mono,
    )


def help_log_verbose(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "on | off"

    return show_usage(
        [
            "@log",
            "verbose",
            f"[{options}]",
        ],
        "verbose logging on/off.",
        mono=mono,
    )


help_functions = {
    "": help_log,
    "list": help_log_list,
    "verbose": help_log_verbose,
}



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_bibclean(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun,install", mono=mono)

    return show_usage(
        [
            "@latex",
            "bibclean",
            f"[{options}]",
            "<path/filename.bib>",
        ],
        "bibclean <path/filename.bib>.",
        mono=mono,
    )


def build_options(mono: bool):
    return "".join(
        [
            "bib=<name>",
            xtra(",dryrun,install,~ps,~pdf", mono=mono),
        ]
    )


def help_build(
    tokens: List[str],
    mono: bool,
) -> str:
    options = build_options(mono)

    return show_usage(
        [
            "@latex",
            "build",
            f"[{options}]",
            "<path/filename.tex>",
        ],
        "build <path/filename.tex>.",
        mono=mono,
    )


def help_install(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun", mono=mono)

    return show_usage(
        [
            "@latex",
            "install",
            f"[{options}]",
        ],
        "install latex.",
        mono=mono,
    )


help_functions = {
    "bibclean": help_bibclean,
    "build": help_build,
    "install": help_install,
}



from typing import List, Dict, Callable, Union

from bluer_options.terminal import show_usage, xtra

build_options = "browse,install,~rm_dist,~upload"


def help_browse(
    tokens: List[str],
    mono: bool,
    plugin_name: str = "bluer_ai",
) -> str:
    options = "token"

    callable = f"{plugin_name} pypi"

    if plugin_name == "bluer_ai":
        options = f"plugin=<plugin-name>,{options}"
        callable = "@pypi"

    return show_usage(
        callable.split(" ")
        + [
            "browse",
            f"[{options}]",
        ],
        f"browse pypi/{plugin_name}.",
        mono=mono,
    )


def help_build(
    tokens: List[str],
    mono: bool,
    plugin_name: str = "bluer_ai",
) -> str:
    options = xtra(build_options, mono=mono)

    callable = f"{plugin_name} pypi"

    if plugin_name == "bluer_ai":
        options = f"{options},plugin=<plugin-name>"
        callable = "@pypi"

    return show_usage(
        callable.split(" ")
        + [
            "build",
            f"[{options}]",
        ],
        f"build pypi/{plugin_name}.",
        mono=mono,
    )


def help_install(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@pypi",
            "install",
        ],
        "install pypi.",
        mono=mono,
    )


def help_functions(
    plugin_name: str = "bluer_ai",
) -> Union[Callable, Dict[str, Union[Callable, Dict]]]:
    return {
        "browse": lambda tokens, mono: help_browse(
            tokens,
            mono=mono,
            plugin_name=plugin_name,
        ),
        "build": lambda tokens, mono: help_build(
            tokens,
            mono=mono,
            plugin_name=plugin_name,
        ),
        "install": help_install,
    }



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_clone(
    tokens: List[str],
    mono: bool,
) -> str:
    # @cp - object object
    options = "".join(
        [
            xtra("~content,cp,~download,", mono=mono),
            "~relate",
            xtra(",~tags,", mono=mono),
            "upload",
        ]
    )

    usage_1 = show_usage(
        [
            "@cp",
            f"[{options}]",
            "[..|<object-1>]",
            "[.|<object-2>]",
        ],
        "copy <object-1> -> <object-2>.",
        mono=mono,
    )

    # @cp - s3 object
    options = "~relate,s3,upload"

    usage_2 = show_usage(
        [
            "@cp",
            f"[{options}]",
            "s3://uri",
            "[.|<object-name>]",
        ],
        "copy s3://uri -> <object-name>.",
        mono=mono,
    )

    return "\n".join(
        [
            usage_1,
            usage_2,
        ]
    )



from typing import List

from bluer_ai import env

from bluer_options.terminal import show_usage, xtra


def help_browse(
    tokens: List[str],
    mono: bool,
) -> str:
    # ---
    options = "cat,id=<job-id>,log"

    usage_1 = show_usage(
        [
            "@batch",
            "browse",
            f"[{options}]",
        ],
        "browse <job-id>.",
        mono=mono,
    )

    # ---
    options = "queue=<queue-name>,status=<status>"

    usage_2 = show_usage(
        [
            "@batch",
            "browse",
            f"[{options}]",
        ],
        "browse <queue-name>.",
        {
            "status: {}".format(
                " | ".join(env.ABCLI_AWS_BATCH_JOB_STATUS_LIST.split(","))
            ): "",
        },
        mono=mono,
    )

    # ---
    options = "queue=list"

    usage_3 = show_usage(
        [
            "@batch",
            "browse",
            f"[{options}]",
        ],
        "browse list of queues.",
        mono=mono,
    )

    return "\n".join(
        [
            usage_1,
            usage_2,
            usage_3,
        ]
    )


def help_cat(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "-"

    return show_usage(
        [
            "@batch",
            "cat",
            f"[{options}]",
            "<job-id>",
        ],
        "cat <job-id>.",
        mono=mono,
    )


def help_eval(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "cat,dryrun,name=<job-name>"

    return show_usage(
        [
            "@batch",
            "eval",
            f"[{options}]",
            "<command-line>",
        ],
        "eval <command-line> in aws batch.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "~count",
            xtra(
                ",dryrun,prefix=<prefix>,status=<status>",
                mono=mono,
            ),
        ]
    )

    return show_usage(
        [
            "@batch",
            "list",
            f"[{options}]",
        ],
        "list aws batch jobs.",
        {
            "status: {}".format(
                " | ".join(env.ABCLI_AWS_BATCH_JOB_STATUS_LIST.split(","))
            ): "",
        },
        mono=mono,
    )


def help_submit(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "cat,dryrun,name=<job-name>"

    return show_usage(
        [
            "@batch",
            "source",
            f"[{options}]",
            "<script-name>",
            "[<args>]",
        ],
        "source <script-name> in aws batch.",
        mono=mono,
    )


help_functions = {
    "browse": help_browse,
    "cat": help_cat,
    "eval": help_eval,
    "list": help_list,
    "submit": help_submit,
}
